package com.ann.utils;

import com.ann.entity.filing.ArchiveDetail;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.time.LocalDate;
import java.util.List;
import java.util.Objects;

/**
 * @Author: LeiJiaXin
 * @Date: 2019/7/24 16:42
 */
@Component
public class FileUtils {

    static final Logger logger = LoggerFactory.getLogger(FileUtils.class);

    // pdf地址
    public static String pdfPath;

    @Value("${file.pdfPath}")
    public void setPdfPath(String pdfPath) {
        FileUtils.pdfPath = pdfPath;
    }

    public static String getPdfPath() {
        return pdfPath;
    }

    // pdf地址
    public static String imagePath;

    @Value("${file.imagePath}")
    public void setImagePath(String imagePath) {
        FileUtils.imagePath = imagePath;
    }

    public static String getImagePath() {
        return imagePath;
    }


    public static File createFile(String fileType, String inpNo, String visitId, String detailId, String tempPath) throws IOException {
        File localFile = null;
        try {
            //生成图片地址
            LocalDate date = LocalDate.now();
            String fileDirName = null;
            if (Objects.equals(fileType, "images")) {
                fileDirName = imagePath+"/" + date.getYear() + "/" + date.getMonthValue() + "/" + inpNo + "_" + visitId;
            } else if (Objects.equals(fileType, "pdfs")) {
                fileDirName = pdfPath + "/" + date.getYear() + "/" + date.getMonthValue() + "/" + inpNo + "_" + visitId;
            }

            //判断是否有该文件夹 否则创建
            File fileDir = new File(fileDirName);
            if (!fileDir.exists()) {
                fileDir.mkdirs();
            }

            String fileName = null;
            if (Objects.equals(fileType, "images")) {
                fileName = fileDir + "/" + tempPath + detailId + ".jpg";
            } else if (Objects.equals(fileType, "pdfs")) {
                fileName = fileDir + "/" + tempPath + detailId + ".pdf";
            }

            localFile = new File(fileName);
            if (!localFile.exists()) {
                localFile.createNewFile();
            }

        } catch (IOException e) {
            throw e;
        }
        return localFile;
    }


    public static File createDir(String fileType, String inpNo, String visitId) throws IOException {
        // ---------------------------此处写死 年月需改善
        LocalDate date = LocalDate.now();
        String fileDirName = null;
        if (Objects.equals(fileType, "images")) {
            fileDirName = imagePath+"/" + date.getYear() + "/" + date.getMonthValue() + "/" + inpNo + "_" + visitId;
        } else if (Objects.equals(fileType, "pdfs")) {
            fileDirName = pdfPath + "/" + date.getYear() + "/" + date.getMonthValue() + "/" + inpNo + "_" + visitId;
        }
        //判断是否有该文件夹 否则创建
        File fileDir = new File(fileDirName);
        if (!fileDir.exists()) {
            fileDir.mkdirs();
        }
        return fileDir;
    }


    public static void deleteImageFile(File file) {
        if (!file.exists()) {
            System.out.println(file.getAbsolutePath() + "文件不存在");
        } else {
            if (file.length() == 0) {
                // 当前文件 大小为0
                File fileParent = new File(file.getParent());
                File[] fileParents = fileParent.listFiles();
                // 如果文件个数大于1 那么已经有其他的报告 只删除本身 否则就删除当前文件以及父级目录
                if (fileParents.length > 1) {
                    // -----------------------------改成log 形式
                    file.delete();
                    logger.error("删除文件为:" + file.getAbsolutePath());
                } else {
                    file.delete();
                    fileParent.delete();
                    logger.error("删除文件为:" + file.getAbsolutePath() + ",以及父文件:" + fileParent.getName());
                }
            }
        }
    }


    public static int deleteImageFile1(File file, int i) {
        if (!file.exists()) {
            System.out.println(file.getAbsolutePath() + "文件不存在");
            return i;
        } else {
            // 当前文件 大小为0
            File fileParent = new File(file.getParent());
            File[] fileParents = fileParent.listFiles();
            // 如果文件个数大于1 那么已经有其他的报告 只删除本身 否则就删除当前文件以及父级目录
            if (fileParents.length > 1) {
                // -----------------------------改成log 形式
                file.delete();
                System.out.println("删除文件为:" + file.getAbsolutePath());
            } else {
                file.delete();
                fileParent.delete();
                System.out.println("删除文件为:" + file.getAbsolutePath() + ",以及父文件:" + fileParent.getName());
            }
            return i++;
        }
    }

    public static File getImageFileByByteArray(byte[] bfile, File file) throws Exception {
        BufferedOutputStream bos = null;
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(file);
            bos = new BufferedOutputStream(fos);
            bos.write(bfile);
            bos.flush();

            return file;
        } catch (Exception e) {
            FileUtils.deleteImageFile(file);
            logger.error("出错咯！错误信息:" + e);
            throw e;
        } finally {
            if (bos != null) {
                try {
                    bos.close();
                } catch (IOException e1) {
                    FileUtils.deleteImageFile(file);
                    logger.error("出错咯！错误信息:" + e1);
                    throw e1;
                }
            }
            if (fos != null) {
                try {
                    fos.close();
                } catch (IOException e1) {
                    FileUtils.deleteImageFile(file);
                    logger.error("出错咯！错误信息:" + e1);
                    throw e1;
                }
            }
        }
    }


    public static void main(String[] args) throws ClassNotFoundException {
        String address = "ftp://10.6.0.155/2019-10-21/000581031700_1_000581031700_0_1_JHR06.00.02_34_2_0.pdf";
        String a = address.substring(0, address.indexOf(".pdf"));
        String emrPath = a.substring(a.lastIndexOf("_") + 1, a.length());

        System.out.println(emrPath);
    }


    public static void haha(List<ArchiveDetail> archiveDetails) {
        for (ArchiveDetail archiveDetail: archiveDetails) {
            String pdfPath = archiveDetail.getPdfPath();
            File file = new File(pdfPath);
            if (!file.exists()) {
                logger.error("masterId为:"+archiveDetail.getMasterID()+",id为:"+archiveDetail.getId()+",文件名为："+pdfPath+",类型为:"+archiveDetail.getAssortID());
            }
        }
    }

}
