package com.ann.utils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.SocketTimeoutException;
import java.util.Properties;
import java.util.Vector;


import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.SftpException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @Author: LeiJiaXin
 * @Date: 2019/9/2 15:22
 */
public class JSchUtils {

    static final Logger logger = LoggerFactory.getLogger(JSchUtils.class);

    private static JSch jsch;
    private static Session session = null;
    private static Channel channel = null;

    /**
     * 连接到指定的IP
     *
     * @throws JSchException
     */
    public static ChannelSftp connect(String ftpUserName, String ftpPassword, String ftpHost, int ftpPort) throws Exception {
        jsch = new JSch();// 创建JSch对象
        session = jsch.getSession(ftpUserName, ftpHost, ftpPort);// 根据用户名、主机ip、端口号获取一个Session对象
        session.setPassword(ftpPassword);// 设置密码

        Properties config = new Properties();
        config.put("StrictHostKeyChecking", "no");
        session.setConfig(config);// 为Session对象设置properties
        session.setTimeout(1000 * 30);// 设置超时
        session.connect();// 通过Session建立连接
        logger.info("Session connected.");
        channel = session.openChannel("sftp"); // 打开SFTP通道
        channel.connect(); // 建立SFTP通道的连接
        logger.info("Connected successfully to ftpHost = " + ftpHost + ",as ftpUserName = " + ftpUserName);
        return (ChannelSftp) channel;
    }

    /**
     * 关闭连接
     */
    public static void close() {
        if (channel != null) {
            channel.disconnect();
            logger.info("关闭channel成功");
        }
        if (session != null) {
            session.disconnect();
            logger.info("关闭session成功");
        }
    }

    /**
     * 下载文件
     *
     * @param src
     * @param dst
     * @throws JSchException
     * @throws SftpException
     */
    public static void download(String src, String dst) throws Exception {
        // src linux服务器文件地址，dst 本地存放地址
        ChannelSftp channelSftp = (ChannelSftp) session.openChannel("sftp");
        channelSftp.connect();
        channelSftp.get(src, dst);
        logger.info("下载文件：" + src + "成功");
        channelSftp.quit();
    }



    public static void main(String[] args) {
        try {
            // 1.连接到指定的服务器
            connect("zingicu", "zingicu123", "10.6.0.241", 22);
            // 2.下载文件
            download("fallSheets/徐晓峰-988874-2020-02-25 - 2020-03-21.pdf", "D:\\pdfs");
            // 4.关闭连接
            close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
