package com.ann.utils;

import com.ann.entity.filing.ArchiveDetail;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Image;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.PdfWriter;
import org.apache.commons.net.ftp.FTPClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sun.misc.BASE64Decoder;
import sun.net.ftp.FtpClient;

import java.io.*;
import java.time.LocalDate;

/**
 * @Author: LeiJiaXin
 * @Date: 2019/7/11 22:32
 */
public class PdfUtils {

    static final Logger logger = LoggerFactory.getLogger(PdfUtils.class);

    /**
     * 生成pdf文件
     *
     * @param imageFile
     * @param pdfFile   需要转换的图片路径的数组
     */
    public static String imagesToPdf(File imageFile, File pdfFile) throws IOException, DocumentException {
        String fileName = null;
        try {
            // ---------------------------此处写死 年月需改善
            // 第一步：创建一个document对象。
            Document document = new Document();
            document.setMargins(0, 0, 0, 0);
            // 第二步：
            // 创建一个PdfWriter实例，
            PdfWriter.getInstance(document, new FileOutputStream(pdfFile));
            // 第三步：打开文档。
            document.open();
            // 第四步：在文档中增加图片。
            if (imageFile.getName().endsWith(".bmp")
                    || imageFile.getName().endsWith(".jpg")
                    || imageFile.getName().endsWith(".jpeg")
                    || imageFile.getName().endsWith(".gif")
                    || imageFile.getName().endsWith(".png")) {
                String imagesPath = imageFile.getAbsolutePath();
                Image img = Image.getInstance(imagesPath);
                img.setAlignment(Image.ALIGN_CENTER);
                // 根据图片大小设置页面，一定要先设置页面，再newPage（），否则无效
                document.setPageSize(new Rectangle(img.getWidth(), img.getHeight()));
                document.newPage();
                document.add(img);
            }
            // 第五步：关闭文档。
            document.close();
            //返回pdf路径
            fileName = pdfFile.getAbsolutePath();
        } catch (Exception e) {
            throw e;
        }
        return fileName;
    }


    /**
     * 生成pdf文件
     *
     * @param pdfFile 需要转换的图片路径的数组
     */
    public static String imageDirToPdf(String imageFiles, File pdfFile) throws IOException, DocumentException {
        String fileName = null;
        try {
            // 第一步：创建一个document对象。
            Document document = new Document();
            document.setMargins(0, 0, 0, 0);
            // 第二步：
            // 创建一个PdfWriter实例，
            PdfWriter.getInstance(document, new FileOutputStream(pdfFile));
            // 第三步：打开文档。
            document.open();

            // 多个文件 地址用;分开
            imageFiles = imageFiles.substring(0, imageFiles.length() - 1);
            String[] fileList = imageFiles.split(";");
            for (String fileStr : fileList) {
                File imageFile = new File(fileStr);
                //File imageFile = new File(dirFile.getPath(),fileStr);
                // 第四步：在文档中增加图片。
                if (imageFile.getName().endsWith(".bmp")
                        || imageFile.getName().endsWith(".jpg")
                        || imageFile.getName().endsWith(".jpeg")
                        || imageFile.getName().endsWith(".gif")
                        || imageFile.getName().endsWith(".png")) {
                    String imagesPath = imageFile.getAbsolutePath();
                    Image img = Image.getInstance(imagesPath);
                    img.setAlignment(Image.ALIGN_CENTER);
                    // 根据图片大小设置页面，一定要先设置页面，再newPage（），否则无效
                    document.setPageSize(new Rectangle(img.getWidth(), img.getHeight()));
                    document.newPage();
                    document.add(img);
                }
            }
            // 第五步：关闭文档。
            document.close();
            //返回pdf路径
            fileName = pdfFile.getAbsolutePath();
        } catch (Exception e) {
            throw e;
        }
        return fileName;
    }

    /**
     *  将base64编码转换成PDF
     *  @param base64String
     *  1.使用BASE64Decoder对编码的字符串解码成字节数组
     *  2.使用底层输入流ByteArrayInputStream对象从字节数组中获取数据；
     *  3.建立从底层输入流中读取数据的BufferedInputStream缓冲输出流对象；
     *  4.使用BufferedOutputStream和FileOutputSteam输出数据到指定的文件中
     */
    public static void base64StringToPDF(String base64String, String pdfPath/*File file*/){
        File file = new File(pdfPath);// 将原来参数修改为字符串
        BASE64Decoder decoder = new BASE64Decoder();
        BufferedInputStream bin = null;
        FileOutputStream fout = null;
        BufferedOutputStream bout = null;
        try {
            //将base64编码的字符串解码成字节数组
            byte[] bytes = decoder.decodeBuffer(base64String);
            //创建一个将bytes作为其缓冲区的ByteArrayInputStream对象
            ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
            //创建从底层输入流中读取数据的缓冲输入流对象
            bin = new BufferedInputStream(bais);
            //创建到指定文件的输出流
            fout  = new FileOutputStream(file);
            //为文件输出流对接缓冲输出流对象
            bout = new BufferedOutputStream(fout);

            byte[] buffers = new byte[1024];
            int len = bin.read(buffers);
            while(len != -1){
                bout.write(buffers, 0, len);
                len = bin.read(buffers);
            }
            //刷新此输出流并强制写出所有缓冲的输出字节，必须这行代码，否则有可能有问题
            bout.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                bout.close();
                fout.close();
                bin.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
